#!/usr/bin/env python3
"""
flare application packager and build tool
"""

import os, shutil, json, argparse, pathlib, fnmatch, watchgod, python_minifier, compileall, tempfile

ignore_patterns = [
    "flare/assets/*",
    "flare/docs/*",
    "flare/examples/*",
    "flare/test/*",
    "flare/tools/*",
    "gen-*"
]


def copySourcePy(source, target):
    """Copy python source files to destination respecting the folder structure."""
    shutil.copytree(source, target, dirs_exist_ok=True)
    cleanSources(target)
    copyflareAssets(source, target)
    copypackageAssets(source, target)


def cleanSources(target):
    cwd = os.getcwd()
    absTarget = os.path.join(cwd, target)

    for folder in ["bin", "docs", "examples", "scripts", "test"]:
        target = os.path.join(absTarget, "flare", folder)
        if os.path.exists(target):
            shutil.rmtree(target)


def minifyPy(target):
    """Minifies all .py-files and strips comments and documentation."""
    for root, _, filenames in os.walk(target):
        for filename in filenames:
            filename = os.path.join(root, filename)

            if filename.endswith(".py"):
                with open(filename, "r+") as f:
                    code = f.read()
                    f.seek(0)
                    f.truncate()
                    f.write(python_minifier.minify(code, remove_literal_statements=True))

def compilePy(target):
    """Compiles py files to pyc and removes all py files at the end."""
    compileall.compile_dir(target, force=True, legacy=True, quiet=1)  # fixme: This does not work when local Python is 3.9.7 but Pyodide is 3.9.5...

    # Remove all source files from target folder
    for root, _, filenames in os.walk(target):
        for filename in filenames:
            filename = os.path.join(root, filename)

            if filename.endswith(".py"):
                os.remove(filename)

    # Update files.json
    generateFilesJson(target, ".pyc")


def zipPy(target, packagename):
    """Zips all files in target folder."""
    tmp = tempfile.mkdtemp()

    to_drop = [packagename]

    for entry in os.scandir(target):
        if entry.name == "flare":
            # special case... move flare/flare to just flare
            flare_tmp = tempfile.mkdtemp()
            shutil.move(os.path.join(target, "flare", "flare"), flare_tmp)
            shutil.rmtree(os.path.join(target, "flare"))
            shutil.move(os.path.join(flare_tmp, "flare"), target)
            to_drop.append("flare")
            continue
        elif entry.name == "packages":
            to_drop.append(entry.name)
            continue

        srcname = os.path.join(target, entry.name)
        dstname = os.path.join(tmp, entry.name)
        shutil.move(srcname, dstname)

    shutil.move(tmp, os.path.join(target, packagename))
    shutil.make_archive(os.path.join(target, "files"), "zip", target, verbose=True)

    for drop in to_drop:
        shutil.rmtree(os.path.join(target, drop))


def copyAssets(source, target):
    """Copy assets to targetfolder.

    if flare exists copy assets first, so that they can be replaced by project
    """
    flarefolder = os.path.join(source, "flare", "assets")
    if os.path.exists(flarefolder):
        shutil.copytree(
            flarefolder, os.path.join(target, "public", "flare"), dirs_exist_ok=True
        )

    assetfolder = os.path.join(source, "public")
    if os.path.exists(assetfolder):
        shutil.copytree(assetfolder, os.path.join(target, "public"), dirs_exist_ok=True)

    for i in os.listdir(source):
        if i.endswith((".html", ".js", ".webmanifest")):
            shutil.copyfile(os.path.join(source, i), os.path.join(target, i))


def copyflareAssets(source, target):
    flarefolder = os.path.join(source, "flare")
    if os.path.exists(flarefolder):
        shutil.copy(
            os.path.join(flarefolder, "flare", "files.json"),
            os.path.join(target, "flare", "flare", "files.json"),
        )

def copyWebworkerScripts(source, target):
    flarefolder = os.path.join( source, "flare", "flare", "webworker")
    if os.path.exists(flarefolder):
        shutil.copytree(
            flarefolder, os.path.join(target, "webworker"), dirs_exist_ok=True
        )

    appfolder = os.path.join(source, "webworker")
    if os.path.exists(appfolder):
        shutil.copytree(
            appfolder, os.path.join(target, "webworker"), dirs_exist_ok=True
        )


def copypackageAssets(source, target):
    packagesFolder = os.path.join(source, "packages")
    if os.path.exists(packagesFolder):
        shutil.copy(
            os.path.join(packagesFolder, "files.json"),
            os.path.join(target, "packages", "files.json"),
        )


def clearTarget(target):
    """Clear target folder."""
    if not os.path.exists(target):
        os.makedirs(target)
    else:
        shutil.rmtree(target)


def generateFilesJson(source, ext=".py"):
    """Walks over the source app directory hierarchy and collects all _wanted_ and _needed_ python files.
    This should work for all of Linux, MacOS and Windows and uses posix compliant path structure.
    """
    files = []

    for root, _, filenames in os.walk(source):
        root = os.path.relpath(root, source)

        for filename in filenames:
            filename = str(os.path.join(root, filename)).removeprefix("./")

            if filename.endswith(ext) and not any([fnmatch.fnmatch(filename, pat) for pat in ignore_patterns]):
                files.append(filename)
                #print(files[-1])

    with open(os.path.join(source, "files.json"), "w") as outputFile:
        json.dump(sorted(files), outputFile, indent=2)
        print("", file=outputFile)  # append line break


def main():
    # parse command-line arguments
    ap = argparse.ArgumentParser(
        description="Flare application packager and build tool"
    )

    ap.add_argument("-s", "--source", required=True, type=pathlib.Path,
                    help="Path to source folder of the flare application")
    ap.add_argument("-t", "--target", required=True, type=pathlib.Path,
                    help="Path to output folder of the packaged flare application")

    ap.add_argument("-n", "--name", type=str, help="Name of the target package", default="app")
    ap.add_argument("-m", "--minify", help="Minify source by removing docstrings", action="store_true", default=False)
    ap.add_argument("-c", "--compile", help="Compile into pre-compiled .PYC-files", action="store_true", default=False)
    ap.add_argument("-z", "--zip", help="Create zipped package to decreased number of download requests",
                    action="store_true", default=False)
    ap.add_argument("-w", "--watch", help="Run in watch mode, re-compile target on any changes in source",
                    action="store_true", default=False)

    args = ap.parse_args()

    if args.source == args.target:
        raise ValueError("You may not set source and target to the same directories")

    if args.watch:
        print("starting initial build")

    os.chdir(os.environ.get("PROJECT_WORKSPACE", "."))

    # Clear target first
    clearTarget(args.target)

    # Copy sources
    copySourcePy(args.source, args.target)

    if args.minify:
        # Minify copied sources
        minifyPy(args.target)

    if args.compile:
        # Turn PY into pre-compiled PYC files
        compilePy(args.target)
    else:
        # If this is not wanted, generate ordinary files.json
        generateFilesJson(args.target)

    if args.zip:
        # Compress target into zip archive
        zipPy(args.target, args.name)

    # Copy and handle webworker scripts separately
    copyWebworkerScripts(args.source, args.target)

    if args.minify:
        minifyPy(os.path.join(args.target, "webworker"))
    #if args.compile:
    #    compilePy(os.path.join(args.target,"webworker"))

    # Copy further assets
    copyAssets(args.source, args.target)

    if args.watch:
        print("watching for changes...")

        for changes in watchgod.watch(args.source, watcher_cls=watchgod.PythonWatcher):
            changes = list(changes)
            filename = os.path.relpath(changes[0][1], args.source)

            if any([fnmatch.fnmatch(filename, pat) for pat in ignore_patterns]):
                continue

            recreateFilesJson = False

            if changes[0][0] == watchgod.watcher.Change.added:
                print(f"{filename} added")
                recreateFilesJson = True
            elif changes[0][0] == watchgod.watcher.Change.deleted:
                print(f"{filename} deleted")
                recreateFilesJson = True
            else:
                print(f"{filename} modified")

            filepath = changes[0][1].replace(str(args.source) + "/", "")

            if not args.zip:
                if changes[0][0] == watchgod.watcher.Change.deleted:
                    os.remove(os.path.join(args.target, filepath))
                else:
                    # copy changed or added file
                    shutil.copy(changes[0][1], os.path.join(args.target, filepath))

                if recreateFilesJson:
                    shutil.copy(os.path.join(args.source, "files.json"), os.path.join(args.target, "files.json"))

            else:
                clearTarget(args.target)
                copySourcePy(args.source, args.target)

            if recreateFilesJson:
                print("regenerating files.json")
                generateFilesJson(args.target)

            if args.minify:
                minifyPy(args.target)

            if args.compile:
                compilePy(args.target)

            if args.zip:
                zipPy(args.target, args.name)
                copyAssets(args.source, args.target)


if __name__ == "__main__":
    main()



import click


def echo_error(msg):
    """colored cli feedback"""
    click.echo(click.style(msg, fg="red"))


def echo_info(msg):
    """colored cli feedback"""
    click.echo(click.style(msg, fg="cyan"))



#!/usr/bin/env python3
import io, os, sys, json, shutil, argparse, pathlib, zipfile
from urllib.request import urlopen

SUPPORTED=[
    # Full Pyodide releases
    "v0.19.0",
    # Current development version of the Pyodide standard
    "dev",
    # Pyodide-nano is shipped as a zip-file
    #"v0.18.0-nano",
    #"v0.18.1-nano",
]

# Defaults
VERSION = SUPPORTED[0]
CDN = "https://cdn.jsdelivr.net/pyodide"
URL = "{CDN}/{VERSION}/full/{file}"
FILES = [
    "pyodide.asm.data",
    "pyodide.asm.js",
    "pyodide.asm.wasm",
    "pyodide.js",
    "pyodide_py.tar"
]
PACKAGES = [
    "distlib",
    "distutils",
    "micropip",
    "packaging",
    "pyparsing",
    "setuptools"
]

# Parse command line arguments
ap = argparse.ArgumentParser(
    description="Service program to obtain self-hosted, stripped copy of Pyodide from CDN"
)
ap.add_argument(
    "-v", "--pyodide", dest="version", default=VERSION, choices=SUPPORTED, help="Pyodide version to download"
)
ap.add_argument("-p", "--packages", nargs="*", help="Further packages to download")
ap.add_argument("-t", "--target", type=pathlib.Path, default="pyodide", help="Target folder")
args = ap.parse_args()

if is_nano := args.version.endswith("-nano"):
    PACKAGES = []

# Allow to install additional Pyodide pre-built packages by command-line arguments
PACKAGES += (args.packages or [])

if is_nano and PACKAGES:
    raise EnvironmentError("Pyodide-nano does not support additionaly packages currently!")

for package in PACKAGES:
    FILES.extend(
        [
            f"{package}.data",
            f"{package}.js",
        ]
    )

# Remove old target folder first
if os.path.dirname(args.target) not in [".", ".."] and os.path.isdir(args.target):
    sys.stdout.write(f"Removing {args.target}/...")
    sys.stdout.flush()

    shutil.rmtree(args.target)
    print("Done")

sys.stdout.write(f"Creating {args.target}/...")
sys.stdout.flush()

os.mkdir(args.target)

# Write version info
open(os.path.join(args.target, f"""{args.version}"""), "a").close()

print("Done")

print(f"Installing Pyodide {args.version}:")

if is_nano:
    # Pyodide Nano is just downloaded from a ZIP-File
    version = args.version.removeprefix("v").removesuffix("-nano")
    url = f"""https://github.com/phorward/pyodide/releases/download/{version}-nano/pyodide-nano-{version}.zip"""

    # Download ZIP-file into memory
    sys.stdout.write(f">>> {url}...")
    sys.stdout.flush()

    zip = urlopen(url).read()

    print("Done")

    # Unpack ZIP file from memory
    sys.stdout.write(f"Unpacking...")
    sys.stdout.flush()

    zip = zipfile.ZipFile(io.BytesIO(zip))
    zip.extractall(args.target)
    zip.close()

    print("Done")

    print(f"Done installing Pyodide {args.version}")
    sys.exit(0)

# Normal install of CDN version

for file in FILES:
    url = URL.format(file=file, CDN=CDN, VERSION=args.version)
    file = os.path.join(args.target, file)

    sys.stdout.write(f">>> {url}...")
    sys.stdout.flush()

    r = urlopen(url).read()

    with open(file, "wb") as f:
        f.write(r)

    print("Done")

# Patch pyodide.js to only use "/pyodide/"
file = os.path.join(args.target, "pyodide.js")
sys.stdout.write(f"Patching {file}...")
sys.stdout.flush()

with open(file, "r") as f:
    content = f.read()

with open(file, "w") as f:
    f.write(
        content.replace('config.indexURL || "./"', 'config.indexURL || "./pyodide/"')
    )

print("Done")

# Write a minimal packages.json with micropip, setuptools and distlibs pre-installed.
file = os.path.join(args.target, "packages.json")
sys.stdout.write(f"Rewriting {file}...")
sys.stdout.flush()

packages = json.loads(urlopen(URL.format(file="packages.json", CDN=CDN, VERSION=args.version)).read())

for k in list(packages["packages"].keys()):
    if k not in PACKAGES:
        del packages["packages"][k]

with open(file, "w") as f:
    f.write(json.dumps(packages))

print("Done")

print(f"Done installing Pyodide {args.version}")

def main():
    pass



import click, os, shutil
from . import cli, echo_error, get_config,utils


@cli.command(context_settings={"ignore_unknown_options": True})
@click.argument("name", default='develop')
@click.argument("additional_args", nargs=-1)
def release(name, additional_args):
    """create a release build"""

    utils.echo_info("building started...")
    projectConfig = get_config()

    if not os.popen("pyenv versions").read():
        echo_error(f"pyenv not found!")
        return

    if name not in projectConfig:
        echo_error(f"{name} is not a valid config name.")
        return

    conf = projectConfig["default"].copy()
    conf.update(projectConfig[name])

    #get pyodide 0.19. Enforce pyodide
    if not conf.get("pyodide"):
        pyodide_version = "v0.19.0"
    else:
        pyodide_version = conf.get("pyodide")

    os.system(f'get-pyodide -t {conf["distribution_folder"]}/pyodide -v {pyodide_version}')

    #build all flare apps
    if conf.get("flare"):
        if "debug" in additional_args:
            flare_build_type = "debug"
            flare_build_env = ""
        else:
            flare_build_type = "release"
            flare_build_env = "pyenv exec"

            #enforce python 3.9.5
            if "3.9.5" not in os.popen("pyenv versions").read():
                os.system(f'pyenv install 3.9.5')
            os.system("pyenv local 3.9.5")

        for name,flare in conf["flare"].items():
            os.system(f'{flare_build_env} viur flare {flare_build_type} {name}')

        if flare_build_type == "release":
            os.system("pyenv local system")

    #build all other apps and assets
    os.system('viur npm')
    utils.echo_info("building finished!")






import click, os, json
from . import cli, echo_error, get_config, echo_info


@cli.command(context_settings={"ignore_unknown_options": True})
@click.argument("action", type=click.Choice(['app', 'index', 'cron', 'queue']))
@click.argument("name", default='develop')
@click.argument("additional_args", nargs=-1)
def deploy(action, name, additional_args):
    """deploy gcloud app or different yaml files"""
    projectConfig = get_config()

    if name not in projectConfig:
        echo_error(f"{name} is not a valid config name.")
        return

    conf = projectConfig["default"].copy()
    conf.update(projectConfig[name])

    if action == "app":
        # rebuild requirements.txt
        create_req()

        os.system(
            f'gcloud app deploy --project={conf["application_name"]} --version={conf["version"]} --no-promote {" ".join(additional_args)} {conf["distribution_folder"]}')
    else:
        if action not in ["index", "queue", "cron"]:
            echo_error(f"{action} is not a valid action. Valid is app, index, queue, cron")

        os.system(
            f'gcloud app deploy --project={conf["application_name"]} {" ".join(additional_args)} {conf["distribution_folder"]}/{action}.yaml')


def create_req():
    """
    load projects pipenv and build a requirements.txt

    cores requirements.txt cant be validated currently, because of the core does not provide a feature for that
    """

    projectConfig = get_config()
    distFolder = projectConfig["default"]["distribution_folder"]

    if click.confirm(f"Do you want to regenerate the requirements.txt located in the {distFolder}?"):
        os.system(f"pipfile2req  --hashes > {distFolder}/requirements.txt")
        #append setupTools for GCP
        file_object = open(f"{distFolder}/requirements.txt", 'a')
        file_object.write('\nsetuptools==59.5.0 --hash=sha256:6d10741ff20b89cd8c6a536ee9dc90d3002dec0226c78fb98605bfb9ef8a7adf')
        file_object.close()

        echo_info("requirements.txt successfully generated")



import click
import os
from .conf import *
from .version import __version__


@click.group(invoke_without_command=True, no_args_is_help=True)
@click.version_option(__version__)
def cli():
    """Info output"""
    load_config()
    fetch_core_version()


@cli.command()
@click.argument("action", type=click.Choice(['add', 'remove', 'list', 'addFlare']))
def project(action):
    """manage project.json and generate if missing"""
    projectConfig = get_config()
    if action == "add":
        add_to_config()
    elif action == "addFlare":
        _projectconf = add_to_flare_config(projectConfig)
        write_config(_projectconf)
    elif action == "remove":
        remove_from_config()
    elif action == "list":
        click.echo(click.style(json.dumps(projectConfig, indent=4, sort_keys=True), fg="cyan"))


@cli.command()
@click.argument("action", type=click.Choice(['ssl']))
def fix(action):
    if action =="ssl":
        os.system("chmod +x scripts/macos_certificate_fix.command && ./scripts/macos_certificate_fix.command")
        click.echo()




import click, os, shutil
from . import cli, echo_error, get_config


@cli.command(context_settings={"ignore_unknown_options": True})
@click.argument("name", default='develop')
@click.argument("additional_args", nargs=-1)
def npm(name, additional_args):
    """create a release build"""
    projectConfig = get_config()

    if name not in projectConfig:
        echo_error(f"{name} is not a valid config name.")
        return

    conf = projectConfig["default"].copy()
    conf.update(projectConfig[name])


    if additional_args:
        script = additional_args[0]
    else:
        script = "build"

    # start sources build
    os.system(f'cd {conf["sources_folder"]} && npm run {script}')




import click
import os
from .conf import *
from .cli import cli
from .utils import *


@cli.command()
def init():
    """initialize project.json"""

    projectConfig = get_config()
    try:
        f = open(projectConfigFilePath)
        if click.confirm("project.json already exists, do you want to delete it and create a new project.json?"):
            os.remove(projectConfigFilePath)
            create_new_config()
    except:

        if not projectConfig:
            if click.confirm("Do you want to create a new project.json file?"):
                create_new_config()


@cli.command()
@click.argument("name")
@click.pass_context
def create(ctx, name):
    """create a new ViUR project"""
    if os.path.exists(f'./{name}'):
        echo_error(f'"{name}" Folder exists. Please use a different name or remove this folder ./{name}')
        return

    # fetch base project
    os.system(
        f'git clone https://github.com/viur-framework/viur-base.git {name}')
    project_json_path = f'./{name}/project.json'

    # collect project info
    create_new_config(project_json_path)
    projectConfig = load_config(path=project_json_path)
    appname = projectConfig["develop"]['application_name']

    # run clean-base
    os.system(f'cd ./{name} && python3 clean-base.py -A={appname}')

    # run gcloud config
    if click.confirm(f'Do you want to configure "{appname}" as a new gcloud project?'):
        os.system(f'cd ./{name} && ./viur-gcloud-setup.sh {appname}')



import click
import os
import json
from .utils import *

projectConfig = None
projectConfigFilePath = "./project.json"


# functions
@click.pass_context
def create_new_config(ctx, path=None):
    """create new config with project.json default template"""

    if path and os.path.exists(os.path.join(os.path.dirname(path), "deploy")):
        deployFolder = "./deploy"
    else:
        deployFolder = click.prompt('distribution folder', default="./deploy")

    if path and os.path.exists(os.path.join(os.path.dirname(path), "sources")):
        sourcesFolder = "./sources"
    else:
        sourcesFolder = click.prompt('distribution folder', default="./sources")



    _projectconf = {
        "default": {
            "format":"1.0.0",
            "distribution_folder": deployFolder,
            "sources_folder": sourcesFolder,
            "flare": {},
            "vi": "submodule",
            "core": "submodule",
            "pyodide": click.prompt('pyodide', default="v0.19.0")
        },

        "develop": {
            "application_name": click.prompt('application name'),
            "version": click.prompt('develop version name')
        }
    }

    if click.confirm("Do you want to use a prebuild Vi? (default)",
                     default=True,
                     show_default=True):
        write_config(_projectconf, path)
        from .vi import vi
        ctx.invoke(vi)
    else:
        if os.path.exists("./sources/vi"):
            echo_info("Found vi application. Added to projet.json.")
            _projectconf["default"]["flare"].update({
                'vi': {
                    "source": './sources/vi/vi',
                    "target": './deploy/vi'
                }
            })
        elif click.confirm("Do you want to add vi as submodule(default)",
                           default=True,
                           show_default=True):
            os.system(
                f'git submodule add https://github.com/viur-framework/viur-vi sources/vi && git submodule update --init --recursive')
            _projectconf["default"]["flare"].update({
                'vi': {
                    "source": './sources/vi/vi',
                    "target": './deploy/vi'
                }
            })
            _projectconf["default"]["vi"] = f'submodule'

    if click.confirm("Do you want to add additional flare application?"):
        _projectconf = add_to_flare_config(_projectconf)

    write_config(_projectconf, path)


def load_config(path=None):
    """load project.json and write to global projetConfig"""
    global projectConfig

    try:
        if not path:
            path = projectConfigFilePath

        f = open(path, "r")
        projectConfig = json.loads(f.read())
    except:
        echo_error("no project.json found")


    update_config()

    return projectConfig


def write_config(conf, path=None):
    """write current projectConfig dict to project.json"""
    global projectConfig

    projectConfig = conf

    if not path:
        path = projectConfigFilePath

    f = open(path, "w")
    f.write(json.dumps(conf, indent=4, sort_keys=True))
    f.close()


def get_config():
    global projectConfig
    return projectConfig


def add_to_config():
    """add a new config to project.json"""
    global projectConfig
    if not projectConfig:
        return

    projectConfig.update({click.prompt('name'): {
        "application_name": click.prompt('application name'),
        "version": click.prompt('develop version name')
    }})

    write_config(projectConfig)


def remove_from_config():
    """remove a config from project.json"""
    global projectConfig
    configname = click.prompt('name')
    try:
        del projectConfig[configname]
        write_config(projectConfig)
    except:
        raise click.ClickException(click.style(f"{configname} not found", fg="red"))


def add_to_flare_config(projectconf):
    """add a new flare app to project.json"""
    if "default" not in projectconf:
        raise click.ClickException(click.style("default entry is missing", fg="red"))

    if not "flare" in projectconf["default"]:
        projectconf["default"].update({"flare": {}})

    projectconf["default"]["flare"].update({
        click.prompt('name'): {
            "source": click.prompt('source'),
            "target": click.prompt('target')
        }
    })

    return projectconf


def remove_from_flare_config(flareAppName):
    """remove a flare app from project.json"""
    global projectConfig
    try:
        del projectConfig["default"]["flare"][flareAppName]
        write_config(projectConfig)
    except:
        raise click.ClickException(click.style(f"{flareAppName} not found", fg="red"))


def fetch_core_version():
    try:
        result = os.popen('pip list --format=json').read()
        coreVersion = [x for x in json.loads(result) if x["name"] == "viur-core"][0]["version"]

        projectConfig["default"]["core"] = coreVersion
        write_config(projectConfig)
    except:
        if projectConfig and projectConfig["default"]:
            projectConfig["default"]["core"] = "submodule"
            write_config(projectConfig)

def update_config():
    if not projectConfig:
        return 0

    if "format" not in projectConfig["default"]:
        projectConfig["default"]["format"] = "1.0.0"

    if "pyodide" not in projectConfig["default"]:
        projectConfig["default"]["pyodide"] = "v0.19.0"

    # conf updates musst increase format version


    write_config(projectConfig)



import click, os, shutil
from . import cli, echo_error, get_config


@cli.command(context_settings={"ignore_unknown_options": True})
@click.argument("action", type=click.Choice(['release', 'debug', 'watch']))
@click.argument("name", default="")
@click.argument("additional_args", nargs=-1)
def flare(action, name, additional_args):
    """build or watch a flare frontend"""
    projectConfig = get_config()
    additional_args = list(additional_args)

    if action == "release":
        additional_args = ["-mzc"]
    elif action == "debug":
        pass
    elif action == "watch":
        additional_args = ["-w"]

    if not name:
        if len(projectConfig["default"]["flare"]) == 0:
            echo_error(f"No flare app configured")
        else:
            name = list(projectConfig["default"]["flare"].keys())[0]

    elif name not in projectConfig["default"]["flare"]:
        echo_error(f"{name} is not a valid flare app name.")
        return

    conf = projectConfig["default"].copy()

    os.system(
        f'flare -n={name} -s={conf["flare"][name]["source"]} -t={conf["flare"][name]["target"]} {" ".join(additional_args)}')



import click, os, zipfile, shutil, urllib, json
from . import cli, get_config, echo_error, write_config
from urllib.request import urlretrieve


@cli.command(context_settings={"ignore_unknown_options": True})
@click.argument("version", default="latest")
def vi(version):
    """download latest vi or a specific version"""
    projectConfig = get_config()
    distFolder = projectConfig["default"]["distribution_folder"]

    viRepo = "https://github.com/viur-framework/viur-vi"
    viPath = os.path.join(distFolder, "vi")
    tempZipFile = "./vi.zip"

    if version == "latest":
        vibaseUrl = f"{viRepo}/releases/latest/download/viur-vi.zip"
    else:
        vibaseUrl = f"{viRepo}/releases/download/v{version}/viur-vi.zip"

    def get_version_info(version):
        url = "https://api.github.com/repos/viur-framework/viur-vi/releases"
        if version == "latest":
            try:
                resp = json.loads(urllib.request.urlopen(url).read().decode("utf-8"))
                projectConfig["default"]["vi"] = resp[0]["name"]
                write_config(projectConfig)
            except:
                echo_error("Error while fetching version info")
        else:
            projectConfig["default"]["vi"] = f'v{version}'
            write_config(projectConfig)

    get_version_info(version)

    def step_label(step):
        if step == 1:
            return f"downloading {version} vi..."
        elif step == 2:
            return f"remove old vi..."
        elif step == 3:
            return f"extracting new vi..."
        elif step == 4:
            return f"success!"

    with click.progressbar([1, 2, 3, 4], label="updating vi...", item_show_func=step_label) as bar:
        for element in bar:
            if element == 1:
                urlretrieve(vibaseUrl, tempZipFile)
            elif element == 2:
                if os.path.exists(viPath):
                    shutil.rmtree(viPath)
            elif element == 3:
                with zipfile.ZipFile(tempZipFile) as zip_f:
                    zip_f.extractall(viPath)
            elif element == 4:
                os.remove(tempZipFile)
                bar.label = "updated successful"



import setuptools
import os
from subprocess import call
from setuptools.command.install import install
from setuptools.command.develop import develop




class PostDevelopCommand(develop):
    """Post-installation for development mode."""
    def run(self):
        develop.run(self)
        call(['./postinstall.sh'])

class PostInstallCommand(install):
    """Post-installation for installation mode."""
    def run(self):
        install.run(self)
        call(['./postinstall.sh'])


setuptools.setup()



import click, os, shutil, subprocess
from . import cli, echo_error, get_config


@cli.command(context_settings={"ignore_unknown_options": True})
@click.argument("name", default='develop')
@click.argument("additional_args", nargs=-1)
def run(name, additional_args):
    """start your application locally"""
    projectConfig = get_config()

    if name not in projectConfig:
        echo_error(f"{name} is not a valid config name.")
        return

    conf = projectConfig["default"].copy()
    conf.update(projectConfig[name])

    os.system(f'app_server -A={conf["application_name"]} {conf["distribution_folder"]} {" ".join(additional_args)}')


@cli.command()
def env():
    """check local Environment"""
    valid_icon = "\U00002714"
    failed_icon = "\U0000274C"

    projectConfig = get_config()

    click.echo(f"Project Info:\n--------------------------------")
    try:
        click.echo(f'Vi: {projectConfig["default"]["vi"]}')
        click.echo(f'Core: {projectConfig["default"]["core"]}')
        click.echo(f'Pyodide: {projectConfig["default"]["pyodide"]}')
        click.echo(f'format: {projectConfig["default"]["format"]}')
    except:
        echo_error("Error while collecting viur info")
    click.echo(f"\nCurrent Environment:\n--------------------------------")

    # viur-cli
    if shutil.which("viur"):
        app_server_version = subprocess.check_output(['viur', '--version']).decode("utf-8")
        click.echo(f"{valid_icon} {app_server_version}")
    else:
        click.echo(f"{failed_icon} ViUR-CLI")

    # app_server
    if shutil.which("app_server"):
        app_server_version = subprocess.check_output(['app_server', '-V']).decode("utf-8")
        click.echo(f"{valid_icon} {app_server_version}")
    else:
        click.echo(f"{failed_icon} app_server")

    # git
    if shutil.which("git"):
        git_version = subprocess.check_output(['git', '--version']).decode("utf-8")
        click.echo(f"{valid_icon} {git_version}")
    else:
        click.echo(f"{failed_icon}")

    # python3
    if shutil.which("python3"):
        npm_version = subprocess.check_output(['python3', '-V']).decode("utf-8")
        click.echo(f"{valid_icon} python3 > {npm_version}")
    else:
        click.echo(f"{failed_icon}")

    # python
    if shutil.which("python"):
        npm_version = subprocess.check_output(['python', '-V']).decode("utf-8")
        click.echo(f"{valid_icon} python > {npm_version}")
    else:
        click.echo(f"{failed_icon}")

    # python3
    if shutil.which("pyenv"):
        pyenv_version = subprocess.check_output(['pyenv', '--version']).decode("utf-8")
        click.echo(f"{valid_icon} {pyenv_version}")
    else:
        click.echo(f"{failed_icon}")

    # npm
    if shutil.which("npm"):
        npm_version = subprocess.check_output(['npm', '-v']).decode("utf-8")
        click.echo(f"{valid_icon} npm {npm_version}")
    else:
        click.echo(f"{failed_icon} npm")

    # node
    if shutil.which("node"):
        npm_version = subprocess.check_output(['node', '-v']).decode("utf-8")
        click.echo(f"{valid_icon} node {npm_version}")
    else:
        click.echo(f"{failed_icon} node")

    # pnpm
    if shutil.which("pnpm"):
        npm_version = subprocess.check_output(['pnpm', '-v']).decode("utf-8")
        click.echo(f"{valid_icon} pnpm {npm_version}")
    else:
        click.echo(f"{failed_icon} pnpm (optional)")

    # gcloud
    if shutil.which("gcloud"):
        gcloud_version = subprocess.check_output(['gcloud', '-v']).decode("utf-8").split("\n\n")[0]
        versionList = []
        for line in gcloud_version.split("\n"):
            if not line:
                continue
            if not line.startswith("Google Cloud SDK"):
                line = " - " + line
            versionList.append(line)
        versionString = '\n'.join(versionList)
        click.echo(f"{valid_icon} {versionString}")
    else:
        click.echo(f"{failed_icon} gcloud")
